package cn.sh.stc.sict.cloud.common.security.service;

import lombok.Getter;
import lombok.Setter;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.User;

import java.util.Collection;

/**
 * @Description 扩展用户信息
 * @Author
 * @Date
 */
public class SictUser extends User {

    @Getter
    @Setter
    private Long id;

    @Getter
    @Setter
    private String phone;

    @Getter
    @Setter
    private String name;

    @Getter
    @Setter
    private String openId;

    @Getter
    @Setter
    private String appId;

    /**
     * 医院公众号登陆的用户包含医院代码
     */
    @Getter
    @Setter
    private String hospitalCode;

    @Getter
    @Setter
    private String xszyUserInfo;

    @Getter
    @Setter
    private transient String token;

    /**
     * Construct the <code>User</code> with the details required by
     * {@link DaoAuthenticationProvider}.
     *
     * @param id                    用户ID
     * @param hospitalCode          医院代码
     * @param username              the username presented to the
     *                              <code>DaoAuthenticationProvider</code>
     * @param password              the password that should be presented to the
     *                              <code>DaoAuthenticationProvider</code>
     * @param enabled               set to <code>true</code> if the user is enabled
     * @param accountNonExpired     set to <code>true</code> if the account has not expired
     * @param credentialsNonExpired set to <code>true</code> if the credentials have not
     *                              expired
     * @param accountNonLocked      set to <code>true</code> if the account is not locked
     * @param authorities           the authorities that should be granted to the caller if they
     *                              presented the correct username and password and the user is enabled. Not null.
     * @throws IllegalArgumentException if a <code>null</code> value was passed either as
     *                                  a parameter or as an element in the <code>GrantedAuthority</code> collection
     */
    public SictUser(Long id,
                    String phone,
                    String openId,
                    String appId,
                    String hospitalCode,
                    String username,
                    String password,
                    boolean enabled,
                    boolean accountNonExpired,
                    boolean credentialsNonExpired,
                    boolean accountNonLocked,
                    Collection<? extends GrantedAuthority> authorities,
                    String xszyUserInfo) {
        super(username, password, enabled, accountNonExpired, credentialsNonExpired, accountNonLocked, authorities);
        this.id = id;
        this.phone = phone;
        this.openId = openId;
        this.appId = appId;
        this.hospitalCode = hospitalCode;
        this.xszyUserInfo = xszyUserInfo;
    }
}
